@@ -312,7 +312,6 @@ int SSL_CTX_use_certificate_chain(SSL_CTX *ctx, const char *file,
  * does client authentication and verifies the certificate chain.
  */
 
-
 int SSL_callback_SSL_verify(int ok, X509_STORE_CTX *ctx)
 {
    /* Get Apache context back through OpenSSL context */
@@ -324,6 +323,7 @@ int SSL_callback_SSL_verify(int ok, X509_STORE_CTX *ctx)
     int errdepth = X509_STORE_CTX_get_error_depth(ctx);
     int verify   = con->ctx->verify_mode;
     int depth    = con->ctx->verify_depth;
+    int ocsp_check_type = con->ctx->no_ocsp_check;
 
 #if defined(SSL_OP_NO_TLSv1_3)
     con->pha_state = PHA_COMPLETE;
@@ -358,25 +358,28 @@ int SSL_callback_SSL_verify(int ok, X509_STORE_CTX *ctx)
 
 #ifdef HAVE_OCSP_STAPLING
     /* First perform OCSP validation if possible */
-    if (ok) {
-        /* If there was an optional verification error, it's not
-         * possible to perform OCSP validation since the issuer may be
-         * missing/untrusted.  Fail in that case.
-         */
-        if (SSL_VERIFY_ERROR_IS_OPTIONAL(errnum)) {
-            X509_STORE_CTX_set_error(ctx, X509_V_ERR_APPLICATION_VERIFICATION);
-            errnum = X509_V_ERR_APPLICATION_VERIFICATION;
-            ok = 0;
-        }
-        else {
-            int ocsp_response = ssl_verify_OCSP(ctx);
-            if (ocsp_response == OCSP_STATUS_REVOKED) {
-                ok = 0 ;
-                errnum = X509_STORE_CTX_get_error(ctx);
+    if (ocsp_check_type == 0) {
+       if (ok) {
+            /* If there was an optional verification error, it's not
+             * possible to perform OCSP validation since the issuer may be
+             * missing/untrusted.  Fail in that case.
+             */
+            if (SSL_VERIFY_ERROR_IS_OPTIONAL(errnum)) {
+                X509_STORE_CTX_set_error(ctx, X509_V_ERR_APPLICATION_VERIFICATION);
+                errnum = X509_V_ERR_APPLICATION_VERIFICATION;
+                ok = 0;
             }
-            else if (ocsp_response == OCSP_STATUS_UNKNOWN) {
-                /* TODO: do nothing for time being */
-                ;
+            else {
+                int ocsp_response = ssl_verify_OCSP(ctx);
+                if (ocsp_response == OCSP_STATUS_REVOKED) {
+                    ok = 0 ;
+                    errnum = X509_STORE_CTX_get_error(ctx);
+                }
+                else if (ocsp_response == OCSP_STATUS_UNKNOWN) {
+                    errnum = X509_STORE_CTX_get_error(ctx);
+                    if (errnum)
+                        ok = 0 ;
+                }
             }
         }
     }
@@ -734,10 +737,13 @@ static char **decode_OCSP_url(ASN1_OCTET_STRING *os, apr_pool_t *p)
     memcpy(ocsp_urls,os->data, len);
     ocsp_urls[len] = '\0';
 
-    if ((response = apr_pcalloc(p, sizeof(char *))) == NULL)
+    if ((response = apr_pcalloc(p, sizeof(char *))) == NULL) {
         return NULL;
-    if (parse_ASN1_Sequence(ocsp_urls, &response, &numofresponses, p))
+    }
+    if (parse_ASN1_Sequence(ocsp_urls, &response, &numofresponses, p) ||
+    		numofresponses ==0) {
         response = NULL;
+    }
     return response;
 }
 
@@ -986,8 +992,11 @@ static OCSP_RESPONSE *get_ocsp_response(apr_pool_t *p, X509 *cert, X509 *issuer,
     int ok = 0;
     apr_socket_t *apr_sock = NULL;
     apr_pool_t *mp;
-
-    if (OCSP_parse_url(url,&hostname, &c_port, &path, &use_ssl) == 0 )
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+    if (OCSP_parse_url(url, &hostname, &c_port, &path, &use_ssl) == 0)
+#else
+    if (OSSL_HTTP_parse_url(url, &use_ssl, NULL, &hostname, &c_port, NULL, &path, NULL, NULL) == 0)
+#endif
         goto end;
 
     if (sscanf(c_port, "%d", &port) != 1)
